@@ -12,6 +12,7 @@ from django.core.exceptions import ImproperlyConfigured
 from django.core.exceptions import ValidationError
 from django.core.exceptions import NON_FIELD_ERRORS
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
+from endless_pagination.paginators import LazyPaginator
 
 
 class HandlerMetaClass(type):
@@ -210,6 +211,10 @@ class BaseHandler(HandlerControlFlow):
 
     # Default item per page, when pagination is requested
     items_per_page = 10
+
+    # Defines whether metadata should be returned when pagination is used.
+    # Generating metadata can, at times, perform costly queries.
+    pagination_metadata = True
     
     def authentication_hook(self):
         """
@@ -553,8 +558,8 @@ class BaseHandler(HandlerControlFlow):
         Args:
             data: Result of the handler's data operation
         Returns:
-            (data_page, total_dict), where ``data_page`` is the page returned
-            and ``total_dict`` is a dictionary containing extra pagination data. 
+            (data_page, metadata), where ``data_page`` is the page returned
+            and ``metadata`` is a dictionary containing extra pagination data. 
             If data is not paginable, or invalid pagination data has been given,
             it returns (data, {})
         """
@@ -769,8 +774,9 @@ class ModelHandler(BaseHandler):
         Args:
             page: Value of querystring parameter ``page``
         Returns:
-            (data_page, {'pages': <total pages>, 'items': <total items>})
-            If for some reason we can't paginate data, returns (data, {})
+            (data_page, {'total_pages': <total pages>, 'total_items': <total items>})
+            If for some reason we can't paginate data, or
+            ``pagination_metadata`` is ``False``, returns (data, {})
         """
         ipp = self.request.GET.get('ipp', None) or self.items_per_page
         try:
@@ -778,15 +784,23 @@ class ModelHandler(BaseHandler):
         except ValueError:
             ipp = self.items_per_page
 
-        paginator = Paginator(data, ipp)
+        if not self.pagination_metadata:
+            paginator = LazyPaginator(data, ipp)
+        else:
+            paginator = Paginator(data, ipp)
 
         try:
             data_page = paginator.page(page)
         except (EmptyPage, PageNotAnInteger, TypeError):
             # TypeError: in case ``data`` is a single model instance
             return data, {}
+        
+        try:
+            metadata = {'total_pages': paginator.num_pages, 'total_items': paginator.count}
+        except NotImplementedError:
+            metadata = {}
 
-        return data_page, {'total_pages': paginator.num_pages, 'total_items': paginator.count}
+        return data_page, metadata
 
 
 """
